# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

from json import dumps
from time import time, sleep

from requests import post
from threading import Thread
from logging import getLogger

from tb_rest_client.api.api_ce import *
from tb_rest_client.models.models_ce import *
from tb_rest_client.configuration import Configuration
from tb_rest_client.api_client import ApiClient

logger = getLogger(__name__)


class RestClientBase(Thread):
    def __init__(self, base_url):
        super().__init__()
        if base_url.startswith("http"):
            self.base_url = base_url
        else:
            self.base_url = "http://" + base_url
        self.token_info = {"token": "", "refreshToken": 0}
        self.api_client = None
        self.username = None
        self.password = None
        self.logged_in = False
        self.stopped = True
        self.configuration = Configuration()
        self.configuration.host = self.base_url

    def run(self):
        self.stopped = False
        while not self.stopped:
            try:
                check_time = time()
                if check_time >= self.token_info["refreshToken"] and self.logged_in:
                    if self.username and self.password:
                        self.login(self.username, self.password)
                    else:
                        logger.error("No username or password provided!")
                sleep(1)
            except Exception as e:
                logger.exception(e)
                break
            except KeyboardInterrupt:
                break

    def stop(self):
        self.stopped = True

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()

    def login(self, username, password):
        """Authorization on the host and saving the toke information"""
        if self.username is None and self.password is None:
            self.username = username
            self.password = password
            self.logged_in = True

        token_json = post(self.base_url + "/api/auth/login", json={"username": username, "password": password}).json()
        token = None
        if isinstance(token_json, dict) and token_json.get("token") is not None:
            token = token_json["token"]
        self.configuration.api_key_prefix["X-Authorization"] = "Bearer"
        self.configuration.api_key["X-Authorization"] = token

        self.api_client = ApiClient(self.configuration)
        self.__load_controllers()

    def get_token(self):
        return self.token_info["token"]

    """Alarm controller endpoints"""

    def get_alarm_by_id(self, alarm_id: AlarmId):
        alarm_id = self.get_id(alarm_id)
        return self.alarm_controller.get_alarm_by_id_using_get(alarm_id)

    def get_alarm_info_by_id(self, alarm_id: AlarmId):
        alarm_id = self.get_id(alarm_id)
        return self.alarm_controller.get_alarm_info_by_id_using_get(alarm_id)

    def delete_alarm(self, alarm_id: AlarmId):
        alarm_id = self.get_id(alarm_id)
        return self.alarm_controller.delete_alarm_using_delete(alarm_id)

    def ack_alarm(self, alarm_id: AlarmId):
        alarm_id = self.get_id(alarm_id)
        return self.alarm_controller.ack_alarm_using_post(alarm_id)

    def clear_alarm(self, alarm_id: AlarmId):
        alarm_id = self.get_id(alarm_id)
        return self.alarm_controller.clear_alarm_using_post(alarm_id)

    def get_alarms(self, entity_id: EntityId, page_size=10, page=0, search_status=None, status=None, text_search=None,
                   sort_property=None, sort_order=None, limit=100000, start_time=None, end_time=None, offset=None,
                   fetch_originator=None):
        return self.alarm_controller.get_alarms_using_get(entity_type=entity_id.entity_type,
                                                          entity_id=entity_id.id,
                                                          page_size=page_size,
                                                          page=page,
                                                          search_status=search_status,
                                                          status=status,
                                                          text_search=text_search,
                                                          sort_property=sort_property,
                                                          sort_order=sort_order,
                                                          start_time=start_time,
                                                          end_time=end_time,
                                                          offset=offset,
                                                          fetch_originator=fetch_originator,
                                                          limit=limit)

    def get_highest_alarm_severity(self, entity_id: EntityId, search_status, status):
        return self.alarm_controller.get_highest_alarm_severity_using_get(entity_type=entity_id.entity_type,
                                                                          entity_id=entity_id.id,
                                                                          search_status=search_status,
                                                                          status=status)

    def save_alarm(self, alarm: Alarm):
        return self.alarm_controller.save_alarm_using_post(alarm)

    """Asset controller endpoints"""

    def get_asset_by_id(self, asset_id: AssetId):
        asset_id = self.get_id(asset_id)
        return self.asset_controller.get_asset_by_id_using_get(asset_id)

    def save_asset(self, asset):
        return self.asset_controller.save_asset_using_post(asset)

    def delete_asset(self, asset_id: AssetId):
        asset_id = self.get_id(asset_id)
        return self.asset_controller.delete_asset_using_delete(asset_id)

    def get_tenant_assets(self, type=None, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None,
                          limit=100000):
        return self.asset_controller.get_tenant_assets_using_get(page_size=str(page_size),
                                                                 page=str(page),
                                                                 type=type,
                                                                 text_search=text_search,
                                                                 sort_property=sort_property,
                                                                 sort_order=sort_order, limit=str(limit))

    def get_tenant_asset(self, asset_name):
        return self.asset_controller.get_tenant_asset_using_get(asset_name)

    def get_customer_assets(self, customer_id: CustomerId, type, page_size=10, page=0, text_search=None,
                            sort_property=None, sort_order=None, limit=100000):
        customer_id = self.get_id(customer_id)
        return self.asset_controller.get_customer_assets_using_get(customer_id=customer_id,
                                                                   page_size=str(page_size),
                                                                   page=str(page),
                                                                   type=type,
                                                                   text_search=text_search,
                                                                   sort_property=sort_property,
                                                                   sort_order=sort_order, limit=str(limit))

    def get_assets_by_ids(self, asset_ids):
        return self.asset_controller.get_assets_by_ids_using_get(asset_ids)

    def find_by_query(self, query):
        if isinstance(query, AssetSearchQuery):
            return self.asset_controller.find_by_query_using_post(query)
        elif isinstance(query, DeviceSearchQuery):
            return self.device_controller.find_by_query_using_post1(query)
        elif isinstance(query, EntityRelationsQuery):
            return self.entity_relation_controller.find_by_query_using_post2(query)
        elif isinstance(query, EntityViewSearchQuery):
            return self.entity_view_controller.find_by_query_using_post3(query)

    def get_asset_types(self):
        return self.asset_controller.get_asset_types_using_get()

    """Audit Log endpoints"""

    def get_audit_logs_by_customer_id(self, customer_id: CustomerId, page_size=10, page=0, text_search=None,
                                      sort_property=None, sort_order=None, limit=100000, start_time=None, end_time=None,
                                      action_types=None):
        customer_id = self.get_id(customer_id)
        return self.audit_log_controller.get_audit_logs_by_customer_id_using_get(customer_id=customer_id,
                                                                                 page_size=str(page_size),
                                                                                 page=str(page),
                                                                                 text_search=text_search,
                                                                                 sort_property=sort_property,
                                                                                 sort_order=sort_order,
                                                                                 limit=str(limit),
                                                                                 start_time=start_time,
                                                                                 end_time=end_time,
                                                                                 action_types=action_types)

    def get_audit_logs_by_user_id(self, user_id: UserId, page_size=10, page=0, text_search=None, sort_property=None,
                                  sort_order=None, limit=100000, start_time=None, end_time=None, action_types=None):
        user_id = self.get_id(user_id)
        return self.audit_log_controller.get_audit_logs_by_user_id_using_get(user_id=user_id,
                                                                             page_size=str(page_size),
                                                                             page=str(page),
                                                                             text_search=text_search,
                                                                             sort_property=sort_property,
                                                                             sort_order=sort_order, limit=str(limit),
                                                                             start_time=start_time,
                                                                             end_time=end_time,
                                                                             action_types=action_types)

    def get_audit_logs_by_entity_id(self, entity_id: EntityId, page_size=10, page=0, text_search=None,
                                    sort_property=None, sort_order=None, limit=100000, start_time=None, end_time=None,
                                    action_types=None):
        return self.audit_log_controller.get_audit_logs_by_entity_id_using_get(entity_type=entity_id.entity_type,
                                                                               entity_id=entity_id.id,
                                                                               page_size=str(page_size),
                                                                               page=str(page),
                                                                               text_search=text_search,
                                                                               sort_property=sort_property,
                                                                               sort_order=sort_order, limit=str(limit),
                                                                               start_time=start_time,
                                                                               end_time=end_time,
                                                                               action_types=action_types)

    def get_audit_logs(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000,
                       start_time=None, end_time=None, action_types=None):
        return self.audit_log_controller.get_audit_logs_using_get(page_size=str(page_size),
                                                                  page=str(page),
                                                                  text_search=text_search,
                                                                  sort_property=sort_property,
                                                                  sort_order=sort_order, limit=str(limit),
                                                                  start_time=start_time,
                                                                  end_time=end_time,
                                                                  action_types=action_types)

    def get_activation_token(self, user_id: UserId):
        user_id = self.get_id(user_id)
        return self.user_controller.get_activation_token_using_get(user_id)

    def get_user(self):
        return self.auth_controller.get_user_using_get()

    def logout(self):
        self.auth_controller.logout_using_post()

    def change_password(self, current_password, new_password):
        change_password_request = '{"currentPassword": "%s","newPassword": "%s"}' % (current_password, new_password)
        self.auth_controller.change_password_using_post(change_password_request)

    def get_user_password_policy(self):
        return self.auth_controller.get_user_password_policy_using_get()

    def check_activation_token(self, user_id: UserId):
        activation_token = self.get_activation_token(user_id)
        return self.auth_controller.check_activate_token_using_get(activation_token)

    def request_reset_password_by_email(self, email: str):
        self.auth_controller.request_reset_password_by_email_using_post('{"email":"%s"}' % email)

    def activate_user(self, user_id: UserId, password):
        user_id = self.get_id(user_id)
        activation_request = {"activateToken": self.get_activation_token(user_id).replace("'", '"'),
                              "password": password}
        return self.auth_controller.activate_user_using_post(activation_request)

    """Component descriptors endpoints"""

    def get_component(self, component_descriptor_class_name):
        return self.component_descriptor_controller.get_component_descriptor_by_clazz_using_get(
            component_descriptor_class_name)

    def get_component_descriptors_by_type(self, component_type):
        return self.component_descriptor_controller.get_component_descriptors_by_type_using_get(component_type)

    def get_component_descriptors_by_types(self, component_types):
        return self.component_descriptor_controller.get_component_descriptors_by_types_using_get(component_types)

    """Customers endpoints"""

    def get_customer_by_id(self, customer_id: CustomerId):
        customer_id = self.get_id(customer_id)
        return self.customer_controller.get_customer_by_id_using_get(customer_id)

    def get_short_customer_info_by_id(self, customer_id: CustomerId):
        customer_id = self.get_id(customer_id)
        return self.customer_controller.get_short_customer_info_by_id_using_get(customer_id)

    def get_customer_title_by_id(self, customer_id: CustomerId):
        customer_id = self.get_id(customer_id)
        return self.customer_controller.get_customer_title_by_id_using_get(customer_id)

    def save_customer(self, customer: Customer):
        return self.customer_controller.save_customer_using_post(customer)

    def delete_customer(self, customer_id: CustomerId):
        customer_id = self.get_id(customer_id)
        self.customer_controller.delete_customer_using_delete(customer_id=customer_id)

    def get_customers(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        return self.customer_controller.get_customers_using_get(page_size=str(page_size),
                                                                page=str(page),
                                                                text_search=text_search,
                                                                sort_property=sort_property,
                                                                sort_order=sort_order, limit=str(limit))

    def get_tenant_customer(self, customer_title):
        return self.customer_controller.get_tenant_customer_using_get(customer_title)

    """Dashboards endpoints"""

    def get_server_time(self):
        return self.dashboard_controller.get_server_time_using_get()

    def get_max_datapoints_limit(self):
        return self.dashboard_controller.get_max_datapoints_limit_using_get()

    def get_dashboard_info_by_id(self, dashboard_id: DashboardId):
        dashboard_id = self.get_id(dashboard_id)
        return self.dashboard_controller.get_dashboard_info_by_id_using_get(dashboard_id)

    def get_dashboard_by_id(self, dashboard_id: DashboardId):
        dashboard_id = self.get_id(dashboard_id)
        return self.dashboard_controller.get_dashboard_by_id_using_get(dashboard_id)

    def save_dashboard(self, dashboard: Dashboard):
        return self.dashboard_controller.save_dashboard_using_post(dashboard)

    def delete_dashboard(self, dashboard_id: DashboardId):
        dashboard_id = self.get_id(dashboard_id)
        self.dashboard_controller.delete_dashboard_using_delete(dashboard_id)

    def get_tenant_dashboards(self, tenant_id=None, page_size=10, page=0, text_search=None, sort_property=None,
                              sort_order=None, limit=100000):
        if tenant_id is not None:
            tenant_id = self.get_id(tenant_id)
            return self.dashboard_controller.get_tenant_dashboards_using_get1(tenant_id=tenant_id,
                                                                              page_size=str(page_size),
                                                                              page=str(page),
                                                                              text_search=text_search,
                                                                              sort_property=sort_property,
                                                                              sort_order=sort_order, limit=str(limit))
        else:
            return self.dashboard_controller.get_tenant_dashboards_using_get(page_size=str(page_size),
                                                                             page=str(page),
                                                                             text_search=text_search,
                                                                             sort_property=sort_property,
                                                                             sort_order=sort_order, limit=str(limit))

    """Device endpoints"""

    def get_device_by_id(self, device_id: DeviceId):
        device_id = self.get_id(device_id)
        return self.device_controller.get_device_by_id_using_get(device_id)

    def save_device(self, device: Device):
        return self.device_controller.save_device_using_post(device)

    def delete_device(self, device_id: DeviceId):
        device_id = self.get_id(device_id)
        return self.device_controller.delete_device_using_delete(device_id)

    def get_device_credentials_by_device_id(self, device_id: DeviceId):
        device_id = self.get_id(device_id)
        return self.device_controller.get_device_credentials_by_device_id_using_get(device_id)

    def save_device_credentials(self, device_credentials: DeviceCredentials):
        return self.device_controller.save_device_credentials_using_post(device_credentials)

    def get_tenant_devices(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None,
                           limit=100000):
        return self.device_controller.get_tenant_devices_using_get(page_size=str(page_size),
                                                                   page=str(page),
                                                                   text_search=text_search,
                                                                   sort_property=sort_property,
                                                                   sort_order=sort_order, limit=str(limit))

    def get_tenant_device(self, device_name):
        return self.device_controller.get_tenant_device_using_get(device_name)

    def get_customer_devices(self, customer_id: CustomerId, type=None, page_size=None, page=None, text_search=None,
                             sort_property=None, sort_order=None, limit=100000):
        customer_id = self.get_id(customer_id)
        return self.device_controller.get_customer_devices_using_get(customer_id=customer_id,
                                                                     page_size=str(page_size),
                                                                     page=str(page),
                                                                     type=type,
                                                                     text_search=text_search,
                                                                     sort_property=sort_property,
                                                                     sort_order=sort_order, limit=str(limit))

    def get_devices_by_ids(self, device_ids):
        self.device_controller.get_devices_by_ids_using_get(device_ids)

    def get_device_types(self):
        return self.device_controller.get_device_types_using_get()

    def claim_device(self, device_name, secret_key, sub_customer_id=None):
        sub_customer_id = self.get_id(sub_customer_id)
        try:
            return self.device_controller.claim_device_using_post(device_name,
                                                                  claim_request='{"secret_key": "%s"}' % (secret_key,),
                                                                  sub_customer_id=sub_customer_id)
        except AttributeError:
            return self.device_controller.claim_device_using_post1(device_name=device_name,
                                                                   claim_request='{"secret_key": "%s"}' % (secret_key,))

    def reclaim_device(self, device_name):
        self.device_controller.re_claim_device_using_delete(device_name)

    """Relation endpoints"""

    def save_relation(self, relation: EntityRelation):
        return self.entity_relation_controller.save_relation_using_post(relation)

    def delete_relation(self, from_id, relation_type, to_id, relation_type_group):
        self.entity_relation_controller.delete_relation_using_delete(from_id=from_id.id,
                                                                     from_type=from_id.entity_type,
                                                                     relation_type=relation_type,
                                                                     to_id=to_id.id,
                                                                     to_type=to_id.entity_type,
                                                                     relation_type_group=relation_type_group)

    def delete_relations(self, entity_id: EntityId):
        self.entity_relation_controller.delete_relations_using_delete(entity_type=entity_id.entity_type,
                                                                      entity_id=entity_id.id)

    def get_relations(self, from_id, relation_type, to_id, relation_type_group):
        return self.entity_relation_controller.get_relation_using_get(from_id=from_id.id,
                                                                      from_type=from_id.entity_type,
                                                                      relation_type=relation_type,
                                                                      to_id=to_id.id,
                                                                      to_type=to_id.entity_type,
                                                                      relation_type_group=relation_type_group)

    def find_by_from(self, from_id, relation_type_group, relation_type=None):
        if relation_type is not None:
            return self.entity_relation_controller.find_by_from_using_get(from_id=from_id.id,
                                                                          from_type=from_id.entity_type,
                                                                          relation_type=relation_type,
                                                                          relation_type_group=relation_type_group)
        else:
            return self.entity_relation_controller.find_by_from_using_get1(from_id=from_id.id,
                                                                           from_type=from_id.entity_type,
                                                                           relation_type_group=relation_type_group)

    def find_info_by_from(self, from_id, relation_type_group):
        return self.entity_relation_controller.find_info_by_from_using_get(from_id=from_id.id,
                                                                           from_type=from_id.entity_type,
                                                                           relation_type_group=relation_type_group)

    def find_by_to(self, to_id, relation_type_group, relation_type=None):
        if relation_type is not None:
            return self.entity_relation_controller.find_by_to_using_get(to_id=to_id.id,
                                                                        to_type=to_id.entity_type,
                                                                        relation_type=relation_type,
                                                                        relation_type_group=relation_type_group)
        else:
            return self.entity_relation_controller.find_by_to_using_get1(to_id=to_id.id,
                                                                         to_type=to_id.entity_type,
                                                                         relation_type_group=relation_type_group)

    def find_info_by_to(self, to_id, relation_type_group):
        return self.entity_relation_controller.find_info_by_to_using_get(to_id=to_id.id,
                                                                         to_type=to_id.entity_type,
                                                                         relation_type_group=relation_type_group)

    def find_info_by_query(self, query):
        return self.entity_relation_controller.find_info_by_query_using_post(query=query)

    """Entity view endpoints"""

    def get_entity_view_by_id(self, entity_view_id: EntityViewId):
        entity_view_id = self.get_id(entity_view_id)
        return self.entity_view_controller.get_entity_view_by_id_using_get(entity_view_id=entity_view_id)

    def save_entity_view(self, entity_view: EntityView):
        return self.entity_view_controller.save_entity_view_using_post(entity_view=entity_view)

    def delete_entity_view(self, entity_view_id: EntityViewId):
        entity_view_id = self.get_id(entity_view_id)
        self.entity_view_controller.delete_entity_view_using_delete(entity_view_id=entity_view_id)

    def get_tenant_entity_view(self, entity_view_name):
        return self.entity_view_controller.get_tenant_entity_view_using_get(entity_view_name=entity_view_name)

    def get_customer_entity_views(self, customer_id: CustomerId, type=None, page_size=10, page=0, text_search=None,
                                  sort_property=None, sort_order=None, limit=100000):
        customer_id = self.get_id(customer_id)
        return self.entity_view_controller.get_customer_entity_views_using_get(customer_id=customer_id,
                                                                               page_size=str(page_size),
                                                                               page=str(page),
                                                                               type=type,
                                                                               text_search=text_search,
                                                                               sort_property=sort_property,
                                                                               sort_order=sort_order, limit=str(limit))

    def get_tenant_entity_views(self, page_size=10, page=0, type=None, text_search=None, sort_property=None,
                                sort_order=None, limit=100000):
        return self.entity_view_controller.get_tenant_entity_views_using_get(page_size=str(page_size),
                                                                             page=str(page),
                                                                             type=type,
                                                                             text_search=text_search,
                                                                             sort_property=sort_property,
                                                                             sort_order=sort_order, limit=str(limit))

    def get_entity_view_types(self):
        return self.entity_view_controller.get_entity_view_types_using_get()

    """ Events endpoints """

    def get_events(self, entity_id: EntityId, tenant_id, event_type=None, page_size=10, page=0, text_search=None,
                   sort_property=None, sort_order=None, limit=100000, start_time=None, end_time=None):
        if event_type is not None:
            return self.event_controller.get_events_using_get(entity_type=entity_id.entity_type,
                                                              entity_id=entity_id.id,
                                                              event_type=event_type,
                                                              tenant_id=tenant_id.id,
                                                              page_size=page_size,
                                                              page=page,
                                                              text_search=text_search,
                                                              sort_property=sort_property,
                                                              sort_order=sort_order, limit=limit,
                                                              start_time=start_time,
                                                              end_time=end_time)
        else:
            return self.event_controller.get_events_using_get1(entity_type=entity_id.entity_type,
                                                               entity_id=entity_id.id,
                                                               tenant_id=tenant_id.id,
                                                               page_size=page_size,
                                                               page=page,
                                                               text_search=text_search,
                                                               sort_property=sort_property,
                                                               sort_order=sort_order, limit=limit,
                                                               start_time=start_time,
                                                               end_time=end_time)

    """RPC endpoints"""

    def handle_one_way_device_rpc_request(self, device_id: DeviceId, request_body):
        device_id = self.get_id(device_id)
        self.rpc_controller.handle_one_way_device_rpc_request_using_post(device_id=device_id.id,
                                                                         request_body=request_body)

    def handle_two_way_device_rpc_request(self, device_id: DeviceId, request_body):
        device_id = self.get_id(device_id)
        return self.rpc_controller.handle_two_way_device_rpc_request_using_post(device_id=device_id,
                                                                                request_body=request_body)

    """Rule chain endpoints"""

    def get_rule_chain_by_id(self, rule_chain_id: RuleChainId):
        rule_chain_id = self.get_id(rule_chain_id)
        return self.rule_chain_controller.get_rule_chain_by_id_using_get(rule_chain_id=rule_chain_id)

    def get_rule_chain_meta_data(self, rule_chain_id: RuleChainId):
        rule_chain_id = self.get_id(rule_chain_id)
        return self.rule_chain_controller.get_rule_chain_meta_data_using_get(rule_chain_id=rule_chain_id)

    def save_rule_chain(self, rule_chain: RuleChain):
        return self.rule_chain_controller.save_rule_chain_using_post(rule_chain=rule_chain)

    def set_root_rule_chain(self, rule_chain_id: RuleChainId):
        rule_chain_id = self.get_id(rule_chain_id)
        return self.rule_chain_controller.set_root_rule_chain_using_post(rule_chain_id=rule_chain_id)

    def save_rule_chain_meta_data(self, rule_chain_meta_data: RuleChainMetaData):
        return self.rule_chain_controller.save_rule_chain_meta_data_using_post(
            rule_chain_meta_data=rule_chain_meta_data)

    def get_rule_chains(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None,
                        limit=100000):
        return self.rule_chain_controller.get_rule_chains_using_get(page_size=str(page_size),
                                                                    page=str(page),
                                                                    text_search=text_search,
                                                                    sort_property=sort_property,
                                                                    sort_order=sort_order, limit=str(limit))

    def delete_rule_chain(self, rule_chain_id: RuleChainId):
        rule_chain_id = self.get_id(rule_chain_id)
        self.rule_chain_controller.delete_rule_chain_using_delete(rule_chain_id)

    def get_latest_rule_node_debug_input(self, rule_node_id: RuleNodeId):
        rule_node_id = self.get_id(rule_node_id)
        return self.rule_chain_controller.get_latest_rule_node_debug_input_using_get(rule_node_id)

    def test_script(self, input_params):
        if isinstance(input_params, dict):
            input_params = dumps(input_params)
        self.rule_chain_controller.test_script_using_post(input_params=input_params)

    """Telemetry endpoints"""

    def get_attribute_keys(self, entity_id: EntityId):
        return self.telemetry_controller.get_attribute_keys_using_get(entity_type=entity_id.entity_type,
                                                                      entity_id=entity_id.id)

    def get_attribute_keys_by_scope(self, entity_id: EntityId, scope):
        return self.telemetry_controller.get_attribute_keys_by_scope_using_get(entity_type=entity_id.entity_type,
                                                                               entity_id=entity_id.id, scope=scope)

    def get_attributes_by_scope(self, entity_id: EntityId, scope, keys=None):
        return self.telemetry_controller.get_attributes_by_scope_using_get(entity_id=entity_id.id,
                                                                           entity_type=entity_id.entity_type,
                                                                           scope=scope, keys=keys)

    def get_timeseries_keys(self, entity_id: EntityId):
        return self.telemetry_controller.get_timeseries_keys_using_get(entity_type=entity_id.entity_type,
                                                                       entity_id=entity_id.id)

    def get_latest_timeseries(self, entity_id: EntityId, keys=None):
        if isinstance(keys, list):
            keys = ",".join(keys)
        return self.telemetry_controller.get_latest_timeseries_using_get(entity_type=entity_id.entity_type,
                                                                         entity_id=entity_id.id, keys=keys)

    def get_timeseries(self, entity_id: EntityId, keys, interval=0, aggregation="NONE", use_strict_data_types=None,
                       start_ts=None, end_ts=None, limit=100):
        if use_strict_data_types is not None:
            return self.telemetry_controller.get_timeseries_using_get(entity_type=entity_id.entity_type,
                                                                      entity_id=entity_id.id,
                                                                      keys=keys,
                                                                      start_ts=start_ts,
                                                                      end_ts=end_ts,
                                                                      interval=interval,
                                                                      limit=str(limit),
                                                                      agg=aggregation,
                                                                      use_strict_data_types=use_strict_data_types)

    def save_device_attributes(self, device_id, scope, request):
        device_id = self.get_id(device_id)
        return self.telemetry_controller.save_device_attributes_using_post(device_id=device_id, scope=scope,
                                                                           request=request)

    def save_entity_attributes_v1(self, entity_id: EntityId, scope, request):
        return self.telemetry_controller.save_entity_attributes_v1_using_post(entity_type=entity_id.entity_type,
                                                                              entity_id=entity_id.id, scope=scope,
                                                                              request=request)

    def save_entity_attributes_v2(self, entity_id: EntityId, scope, request):
        return self.telemetry_controller.save_entity_attributes_v2_using_post(entity_type=entity_id.entity_type,
                                                                              entity_id=entity_id.id, scope=scope,
                                                                              request=request)

    def save_entity_telemetry(self, entity_id: EntityId, scope, request):
        return self.telemetry_controller.save_entity_telemetry_using_post(entity_type=entity_id.entity_type,
                                                                          entity_id=entity_id.id, scope=scope,
                                                                          request_body=request)

    def save_entity_telemetry_with_ttl(self, entity_id: EntityId, scope, ttl, request):
        return self.telemetry_controller.save_entity_telemetry_with_ttl_using_post(entity_type=entity_id.entity_type,
                                                                                   entity_id=entity_id.id, scope=scope,
                                                                                   ttl=ttl, request_body=request)

    def delete_entity_timeseries(self, entity_id: EntityId, keys, delete_all_data_for_keys, start_ts, end_ts,
                                 rewrite_latest_if_deleted):
        return self.telemetry_controller.delete_entity_timeseries_using_delete(entity_type=entity_id.entity_type,
                                                                               entity_id=entity_id.id,
                                                                               keys=keys,
                                                                               delete_all_data_for_keys=delete_all_data_for_keys,
                                                                               start_ts=start_ts,
                                                                               end_ts=end_ts,
                                                                               rewrite_latest_if_deleted=rewrite_latest_if_deleted)

    def delete_entity_attributes(self, id, scope, keys):
        if isinstance(keys, list):
            keys = ",".join(keys)
        if isinstance(id, DeviceId):
            return self.telemetry_controller.delete_entity_attributes_using_delete(id.id, scope, keys)
        elif isinstance(id, EntityId):
            return self.telemetry_controller.delete_entity_attributes_using_delete1(entity_type=id.entity_type,
                                                                                    entity_id=id.id,
                                                                                    scope=scope,
                                                                                    keys=keys)

    """ Tenant endpoints. """

    def get_tenant_by_id(self, tenant_id: TenantId):
        tenant_id = self.get_id(tenant_id)
        return self.tenant_controller.get_tenant_by_id_using_get(tenant_id=tenant_id)

    def save_tenant(self, tenant: Tenant):
        return self.tenant_controller.save_tenant_using_post(tenant=tenant)

    def delete_tenant(self, tenant_id: TenantId):
        tenant_id = self.get_id(tenant_id)
        self.tenant_controller.delete_tenant_using_delete(tenant_id=tenant_id)

    def get_tenants(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        return self.tenant_controller.get_tenants_using_get(page_size=str(page_size),
                                                            page=str(page),
                                                            text_search=text_search,
                                                            sort_property=sort_property,
                                                            sort_order=sort_order, limit=str(limit))

    def get_tenant_admins(self, tenant_id: TenantId, page_size=10, page=0, text_search=None, sort_property=None,
                          sort_order=None, limit=100000):
        tenant_id = self.get_id(tenant_id)
        return self.user_controller.get_tenant_admins_using_get(tenant_id=tenant_id,
                                                                page_size=str(page_size),
                                                                page=str(page),
                                                                text_search=text_search,
                                                                sort_property=sort_property,
                                                                sort_order=sort_order, limit=str(limit))

    """ User endpoints. """

    def get_user_by_id(self, user_id: UserId):
        user_id = self.get_id(user_id)
        return self.user_controller.get_user_by_id_using_get(user_id=user_id)

    def is_user_token_access_enabled(self):
        return self.user_controller.is_user_token_access_enabled_using_get()

    def get_user_token(self, user_id: UserId):
        user_id = self.get_id(user_id)
        return self.user_controller.get_user_token_using_get(user_id=user_id)

    def save_user(self, user: User, send_activation_mail, entity_group_id=None):
        return self.user_controller.save_user_using_post(user=user, send_activation_mail=send_activation_mail,
                                                         entity_group_id=entity_group_id)

    def send_activation_email(self, email):
        self.user_controller.send_activation_email_using_post(email=email)

    def get_activation_link(self, user_id: UserId):
        user_id = self.get_id(user_id)
        return self.user_controller.get_activation_link_using_get(user_id=user_id)

    def delete_user(self, user_id: UserId):
        user_id = self.get_id(user_id)
        self.user_controller.delete_user_using_delete(user_id=user_id)

    def get_customer_users(self, customer_id: CustomerId, page_size=10, page=0, text_search=None, sort_property=None,
                           sort_order=None, limit=100000):
        customer_id = self.get_id(customer_id)
        return self.user_controller.get_customer_users_using_get(customer_id=customer_id,
                                                                 page_size=str(page_size),
                                                                 page=str(page),
                                                                 text_search=text_search,
                                                                 sort_property=sort_property,
                                                                 sort_order=sort_order, limit=str(limit))

    def set_user_credentials_enabled(self, user_id: UserId, user_credentials_enabled):
        user_id = self.get_id(user_id)
        return self.user_controller.set_user_credentials_enabled_using_post(user_id=user_id,
                                                                            user_credentials_enabled=user_credentials_enabled)

    """ Widget endpoints. """

    def get_widgets_budle_by_id(self, widgets_bundle_id: WidgetsBundleId):
        widgets_bundle_id = self.get_id(widgets_bundle_id)
        return self.widgets_bundle_controller.get_widgets_bundle_by_id_using_get(widgets_bundle_id=widgets_bundle_id)

    def save_widgets_bundle(self, widgets_bundle: WidgetsBundle):
        return self.widgets_bundle_controller.save_widgets_bundle_using_post(widgets_bundle=widgets_bundle)

    def delete_widgets_bundle(self, widgets_bundle_id: WidgetsBundleId):
        widgets_bundle_id = self.get_id(widgets_bundle_id)
        self.widgets_bundle_controller.delete_widgets_bundle_using_delete(widgets_bundle_id=widgets_bundle_id)

    def get_widgets_bundles(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None,
                            limit=100000):
        return self.widgets_bundle_controller.get_widgets_bundles_using_get(page_size=str(page_size),
                                                                            page=str(page),
                                                                            text_search=text_search,
                                                                            sort_property=sort_property,
                                                                            sort_order=sort_order, limit=str(limit))

    """ Widget type endpoints"""

    def get_widget_type_by_id(self, widget_type_id: WidgetTypeId):
        widget_type_id = self.get_id(widget_type_id)
        return self.widget_type_controller.get_widget_type_by_id_using_get(widget_type_id=widget_type_id)

    def save_widget_type(self, widget_type: WidgetType):
        return self.widget_type_controller.save_widget_type_using_post(widget_type=widget_type)

    def delete_widget_type(self, widget_type_id: WidgetTypeId):
        widget_type_id = self.get_id(widget_type_id)
        self.widget_type_controller.delete_widget_type_using_delete(widget_type_id=widget_type_id)

    def get_bundle_widget_types(self, is_system, bundle_alias):
        is_system = str(is_system).lower()
        return self.widget_type_controller.get_bundle_widget_types_using_get(is_system=is_system,
                                                                             bundle_alias=bundle_alias)

    def get_widget_type(self, is_system, bundle_alias, alias):
        is_system = str(is_system).lower()
        return self.widget_type_controller.get_widget_type_using_get(is_system=is_system, bundle_alias=bundle_alias,
                                                                     alias=alias)

    def __load_controllers(self):
        self.auth_controller = AuthControllerApi(self.api_client)
        self.admin_controller = AdminControllerApi(self.api_client)
        self.alarm_controller = AlarmControllerApi(self.api_client)
        self.asset_controller = AssetControllerApi(self.api_client)
        self.audit_log_controller = AuditLogControllerApi(self.api_client)
        self.component_descriptor_controller = ComponentDescriptorControllerApi(self.api_client)
        self.customer_controller = CustomerControllerApi(self.api_client)
        self.dashboard_controller = DashboardControllerApi(self.api_client)
        self.device_controller = DeviceControllerApi(self.api_client)
        self.entity_relation_controller = EntityRelationControllerApi(self.api_client)
        self.entity_view_controller = EntityViewControllerApi(self.api_client)
        self.event_controller = EventControllerApi(self.api_client)
        self.queue_controller = QueueControllerApi(self.api_client)
        self.rpc_controller = RpcControllerApi(self.api_client)
        self.rule_chain_controller = RuleChainControllerApi(self.api_client)
        self.telemetry_controller = TelemetryControllerApi(self.api_client)
        self.tenant_controller = TenantControllerApi(self.api_client)
        self.user_controller = UserControllerApi(self.api_client)
        self.widget_type_controller = WidgetTypeControllerApi(self.api_client)
        self.widgets_bundle_controller = WidgetsBundleControllerApi(self.api_client)

    @staticmethod
    def get_type(type):
        return type.entity_type if hasattr(type, "entity_type") else type

    @staticmethod
    def get_id(id):
        return id.id if hasattr(id, "id") else id



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

# flake8: noqa
from __future__ import absolute_import

# import models_ce into model package

from .admin_settings import AdminSettings
from .admin_settings_id import AdminSettingsId
from .alarm import Alarm
from .alarm_id import AlarmId
from .alarm_info import AlarmInfo
from .asset import Asset
from .asset_id import AssetId
from .asset_search_query import AssetSearchQuery
from .attributes_entity_view import AttributesEntityView
from .audit_log import AuditLog
from .audit_log_id import AuditLogId
from .blob_entity_id import BlobEntityId
from .claim_request import ClaimRequest
from .component_descriptor import ComponentDescriptor
from .component_descriptor_id import ComponentDescriptorId
from .converter_id import ConverterId
from .customer import Customer
from .customer_id import CustomerId
from .dashboard import Dashboard
from .dashboard_id import DashboardId
from .dashboard_info import DashboardInfo
from .deferred_result_response_entity import DeferredResultResponseEntity
from .device import Device
from .device_credentials import DeviceCredentials
from .device_credentials_id import DeviceCredentialsId
from .device_id import DeviceId
from .device_search_query import DeviceSearchQuery
from .entity_group_id import EntityGroupId
from .entity_id import EntityId
from .entity_relation import EntityRelation
from .entity_relation_info import EntityRelationInfo
from .entity_relations_query import EntityRelationsQuery
from .entity_subtype import EntitySubtype
from .entity_type_filter import EntityTypeFilter
from .entity_view import EntityView
from .entity_view_id import EntityViewId
from .entity_view_search_query import EntityViewSearchQuery
from .event import Event
from .event_id import EventId
from .group_permission_id import GroupPermissionId
from .integration_id import IntegrationId
from .node_connection_info import NodeConnectionInfo
from .o_auth2_client_info import OAuth2ClientInfo
from .o_auth2_integration_id import OAuth2IntegrationId
from .relations_search_parameters import RelationsSearchParameters
from .response_entity import ResponseEntity
from .role_id import RoleId
from .rule_chain import RuleChain
from .rule_chain_connection_info import RuleChainConnectionInfo
from .rule_chain_id import RuleChainId
from .rule_chain_meta_data import RuleChainMetaData
from .rule_node import RuleNode
from .rule_node_id import RuleNodeId
from .scheduler_event_id import SchedulerEventId
from .security_settings import SecuritySettings
from .short_customer_info import ShortCustomerInfo
from .telemetry_entity_view import TelemetryEntityView
from .tenant import Tenant
from .tenant_id import TenantId
from .text_page_data_asset import TextPageDataAsset
from .text_page_data_customer import TextPageDataCustomer
from .text_page_data_dashboard_info import TextPageDataDashboardInfo
from .text_page_data_device import TextPageDataDevice
from .text_page_data_entity_view import TextPageDataEntityView
from .text_page_data_rule_chain import TextPageDataRuleChain
from .text_page_data_tenant import TextPageDataTenant
from .text_page_data_user import TextPageDataUser
from .text_page_data_widgets_bundle import TextPageDataWidgetsBundle
from .text_page_link import TextPageLink
from .time_page_data_alarm_info import TimePageDataAlarmInfo
from .time_page_data_audit_log import TimePageDataAuditLog
from .time_page_data_dashboard_info import TimePageDataDashboardInfo
from .time_page_data_event import TimePageDataEvent
from .time_page_link import TimePageLink
from .update_message import UpdateMessage
from .user import User
from .user_id import UserId
from .user_password_policy import UserPasswordPolicy
from .widget_type import WidgetType
from .widget_type_id import WidgetTypeId
from .widgets_bundle import WidgetsBundle
from .widgets_bundle_id import WidgetsBundleId



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

# flake8: noqa
from __future__ import absolute_import

# import models_base into model package

# from tb_rest_client.models.models_base.admin_settings import AdminSettings
# from tb_rest_client.models.models_base.admin_settings_id import AdminSettingsId
# from tb_rest_client.models.models_base.response_entity import ResponseEntity
# from tb_rest_client.models.models_base.alarm import Alarm
# from tb_rest_client.models.models_base.alarm_id import AlarmId
# from tb_rest_client.models.models_base.alarm_info import AlarmInfo
# from tb_rest_client.models.models_base.asset_id import AssetId
# from tb_rest_client.models.models_base.asset_search_query import AssetSearchQuery
# from tb_rest_client.models.models_base.attributes_entity_view import AttributesEntityView
# from tb_rest_client.models.models_base.audit_log_id import AuditLogId
# from tb_rest_client.models.models_base.blob_entity_id import BlobEntityId
# from tb_rest_client.models.models_base.claim_request import ClaimRequest
# from tb_rest_client.models.models_base.component_descriptor_id import ComponentDescriptorId
# from tb_rest_client.models.models_base.converter_id import ConverterId
# from tb_rest_client.models.models_base.customer_id import CustomerId
# from tb_rest_client.models.models_base.role_id import RoleId
# from tb_rest_client.models.models_base.rule_chain import RuleChain
# from tb_rest_client.models.models_base.rule_chain_connection_info import RuleChainConnectionInfo
# from tb_rest_client.models.models_base.rule_chain_id import RuleChainId
# from tb_rest_client.models.models_base.rule_chain_meta_data import RuleChainMetaData
# from tb_rest_client.models.models_base.rule_node import RuleNode
# from tb_rest_client.models.models_base.rule_node_id import RuleNodeId
# from tb_rest_client.models.models_base.scheduler_event_id import SchedulerEventId
# from tb_rest_client.models.models_base.security_settings import SecuritySettings
# from tb_rest_client.models.models_base.short_customer_info import ShortCustomerInfo
# from tb_rest_client.models.models_base.telemetry_entity_view import TelemetryEntityView
# from tb_rest_client.models.models_base.tenant import Tenant
# from tb_rest_client.models.models_base.tenant_id import TenantId
# from tb_rest_client.models.models_base.update_message import UpdateMessage
# from tb_rest_client.models.models_base.user_id import UserId
# from tb_rest_client.models.models_base.user_password_policy import UserPasswordPolicy
# from tb_rest_client.models.models_base.widget_type import WidgetType
# from tb_rest_client.models.models_base.widget_type_id import WidgetTypeId
# from tb_rest_client.models.models_base.widgets_bundle import WidgetsBundle
# from tb_rest_client.models.models_base.widgets_bundle_id import WidgetsBundleId
# from tb_rest_client.models.models_base.dashboard_id import DashboardId
# from tb_rest_client.models.models_base.deferred_result_response_entity import DeferredResultResponseEntity
# from tb_rest_client.models.models_base.device_credentials import DeviceCredentials
# from tb_rest_client.models.models_base.device_credentials_id import DeviceCredentialsId
# from tb_rest_client.models.models_base.device_id import DeviceId
# from tb_rest_client.models.models_base.device_search_query import DeviceSearchQuery
# from tb_rest_client.models.models_base.entity_group_id import EntityGroupId
# from tb_rest_client.models.models_base.entity_id import EntityId
# from tb_rest_client.models.models_base.entity_relations_query import EntityRelationsQuery
# from tb_rest_client.models.models_base.entity_view_id import EntityViewId
# from tb_rest_client.models.models_base.entity_view_search_query import EntityViewSearchQuery
# from tb_rest_client.models.models_base.event import Event
# from tb_rest_client.models.models_base.event_id import EventId
# from tb_rest_client.models.models_base.group_permission_id import GroupPermissionId
# from tb_rest_client.models.models_base.integration_id import IntegrationId
# from tb_rest_client.models.models_base.node_connection_info import NodeConnectionInfo
# from tb_rest_client.models.models_base.o_auth2_client_info import OAuth2ClientInfo
# from tb_rest_client.models.models_base.o_auth2_integration_id import OAuth2IntegrationId

# import models_pe into model package

from .allowed_permissions_info import AllowedPermissionsInfo
from .asset import Asset
from .audit_log import AuditLog
from .blob_entity_info import BlobEntityInfo
from .blob_entity_with_customer_info import BlobEntityWithCustomerInfo
from .component_descriptor import ComponentDescriptor
from .contact_basedobject import ContactBasedobject
from .converter import Converter
from .custom_menu import CustomMenu
from .custom_menu_item import CustomMenuItem
from .custom_translation import CustomTranslation
from .customer import Customer
from .dashboard import Dashboard
from .dashboard_info import DashboardInfo
from .device import Device
from .entity_group import EntityGroup
from .entity_group_info import EntityGroupInfo
from .entity_relation import EntityRelation
from .entity_relation_info import EntityRelationInfo
from .entity_subtype import EntitySubtype
from .entity_type_filter import EntityTypeFilter
from .entity_view import EntityView
from .favicon import Favicon
from .file import File
from .group_permission import GroupPermission
from .group_permission_info import GroupPermissionInfo
from .input_stream import InputStream
from .integration import Integration
from .login_white_labeling_params import LoginWhiteLabelingParams
from .merged_group_permission_info import MergedGroupPermissionInfo
from .merged_group_type_permission_info import MergedGroupTypePermissionInfo
from .merged_user_permissions import MergedUserPermissions
from .page_data_alarm_info import PageDataAlarmInfo
from .page_data_asset import PageDataAsset
from .page_data_audit_log import PageDataAuditLog
from .page_data_blob_entity_with_customer_info import PageDataBlobEntityWithCustomerInfo
from .page_data_contact_basedobject import PageDataContactBasedobject
from .page_data_converter import PageDataConverter
from .page_data_customer import PageDataCustomer
from .page_data_dashboard_info import PageDataDashboardInfo
from .page_data_device import PageDataDevice
from .page_data_entity_view import PageDataEntityView
from .page_data_event import PageDataEvent
from .page_data_integration import PageDataIntegration
from .page_data_role import PageDataRole
from .page_data_rule_chain import PageDataRuleChain
from .page_data_short_entity_view import PageDataShortEntityView
from .page_data_tenant import PageDataTenant
from .page_data_user import PageDataUser
from .page_data_widgets_bundle import PageDataWidgetsBundle
from .palette import Palette
from .palette_settings import PaletteSettings
from .relations_search_parameters import RelationsSearchParameters
from .report_config import ReportConfig
from .resource import Resource
from .role import Role
from .scheduler_event import SchedulerEvent
from .scheduler_event_info import SchedulerEventInfo
from .scheduler_event_with_customer_info import SchedulerEventWithCustomerInfo
from .self_registration_params import SelfRegistrationParams
from .short_entity_view import ShortEntityView
from .sign_up_request import SignUpRequest
from .sign_up_self_registration_params import SignUpSelfRegistrationParams
from .uri import URI
from .url import URL
from .user import User
from .white_labeling_params import WhiteLabelingParams



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

import pprint
import re  # noqa: F401

import six

from tb_rest_client.models.models_ce import Asset


class Asset(Asset):
    """NOTE: This class is auto generated by the swagger code generator program.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {**Asset.swagger_types, "owner_id": "EntityId"}
    attribute_map = {**Asset.attribute_map, "owner_id": "ownerId"}

    def __init__(self, additional_info=None, created_time=None, customer_id=None, id=None, label=None, name=None, owner_id=None, tenant_id=None, type=None):  # noqa: E501
        """Asset - a model defined in Swagger"""  # noqa: E501

        super().__init__(additional_info, created_time, customer_id, id, label, name, tenant_id, type)
        self._owner_id = owner_id

    @property
    def owner_id(self):
        """Gets the owner_id of this Asset.  # noqa: E501


        :return: The owner_id of this Asset.  # noqa: E501
        :rtype: EntityId
        """
        return self._owner_id

    @owner_id.setter
    def owner_id(self, owner_id):
        """Sets the owner_id of this Asset.


        :param owner_id: The owner_id of this Asset.  # noqa: E501
        :type: EntityId
        """

        self._owner_id = owner_id



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

import pprint
import re  # noqa: F401

import six

from tb_rest_client.models.models_ce import ComponentDescriptor

class ComponentDescriptor(ComponentDescriptor):
    def __init__(self, actions=None, clazz=None, configuration_descriptor=None, created_time=None, id=None, name=None, scope=None, type=None):  # noqa: E501
        super().__init(actions, clazz, configuration_descriptor, created_time, id, name, scope, type)
        self._type = None
        if type is not None:
            self.type = type

    @property
    def type(self):
        """Gets the type of this ComponentDescriptor.  # noqa: E501


        :return: The type of this ComponentDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ComponentDescriptor.


        :param type: The type of this ComponentDescriptor.  # noqa: E501
        :type: str
        """
        allowed_values = ["ENRICHMENT", "FILTER", "TRANSFORMATION", "ACTION", "ANALYTICS", "EXTERNAL"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

from tb_rest_client.rest_client_base import *

from tb_rest_client.api.api_ce import *
from tb_rest_client.api.api_pe import *
from tb_rest_client.models.models_pe import *

logger = getLogger(__name__)


class RestClientPE(RestClientBase):
    def __init__(self, base_url):
        super().__init__(base_url)

    def login(self, username, password):
        super(RestClientPE, self).login(username=username, password=password)
        self.__load_controllers()

    """Admin controller endpoints"""

    def get_admin_settings(self, key, system_by_default=False):
        return self.admin_controller.get_admin_settings_using_get(key=key, system_by_default=system_by_default)

    def save_admin_settings(self, admin_settings: AdminSettings):
        return self.admin_controller.save_admin_settings_using_post(admin_settings)

    def send_test_mail(self, admin_settings: AdminSettings):
        return self.admin_controller.send_test_mail_using_post(admin_settings)

    def get_security_settings(self):
        return self.admin_controller.get_security_settings_using_get()

    def save_security_settings(self, security_settings: SecuritySettings):
        return self.admin_controller.save_security_settings_using_post(security_settings)

    def check_updates(self):
        return self.admin_controller.check_updates_using_get()

    """ Dashboard endpoints """

    def get_user_dashboards(self, user_id: UserId, operation, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        user_id = self.get_id(user_id)
        return self.dashboard_controller.get_user_dashboards_using_get(page_size=str(page_size),
                                                                       page=str(page),
                                                                       text_search=text_search,
                                                                       sort_property=sort_property,
                                                                       sort_order=sort_order, limit=str(limit),
                                                                       operation=operation,
                                                                       user_id=user_id)

    def get_group_dashboards(self, entity_group_id: EntityGroupId, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        entity_group_id = self.get_id(entity_group_id)
        return self.dashboard_controller.get_group_dashboards_using_get(entity_group_id=entity_group_id,
                                                                        page_size=str(page_size),
                                                                        page=str(page),
                                                                        text_search=text_search,
                                                                        sort_property=sort_property,
                                                                        sort_order=sort_order, limit=str(limit))

    def get_dasboards_by_ids(self, dashboards_ids):
        if isinstance(dashboards_ids, list):
            dashboards_ids = ",".join(dashboards_ids)
        return self.dashboard_controller.get_dashboards_by_ids_using_get(dashboard_ids=dashboards_ids)

    """ Tenant endpoints """

    def get_tenants_by_ids(self, tenant_ids):
        return self.tenant_controller.get_tenants_by_ids_using_get(tenant_ids=tenant_ids)

    """ Blob Entity endpoints"""

    def get_blob_entity_info(self, blob_entity_id: BlobEntityId):
        blob_entity_id = self.get_id(blob_entity_id)
        return self.blob_entity_controller.get_blob_entity_info_by_id_using_get(blob_entity_id=blob_entity_id)

    def download_blob_entity(self, blob_entity_id: BlobEntityId):
        blob_entity_id = self.get_id(blob_entity_id)
        return self.blob_entity_controller.download_blob_entity_using_get(blob_entity_id=blob_entity_id)

    def delete_blob_entity(self, blob_entity_id: BlobEntityId):
        blob_entity_id = self.get_id(blob_entity_id)
        self.blob_entity_controller.delete_blob_entity_using_delete(blob_entity_id=blob_entity_id)

    def get_blob_entities(self, type, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, start_time=None, end_time=None):
        return self.blob_entity_controller.get_blob_entities_using_get(page_size=page_size,
                                                                       page=page,
                                                                       type=type,
                                                                       text_search=text_search,
                                                                       sort_property=sort_property,
                                                                       sort_order=sort_order,
                                                                       start_time=start_time,
                                                                       end_time=end_time)

    def get_blob_entities_by_ids(self, blob_entity_ids):
        return self.blob_entity_controller.get_blob_entities_by_ids_using_get(blob_entity_ids=blob_entity_ids)

    """ Converter endpoints """

    def get_converter_by_id(self, converter_id: ConverterId):
        converter_id = self.get_id(converter_id)
        return self.converter_controller.get_converter_by_id_using_get(converter_id=converter_id)

    def save_converter(self, converter: Converter):
        return self.converter_controller.save_converter_using_post(converter=converter)

    def get_converters(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None):
        return self.converter_controller.get_converters_using_get(page_size=str(page_size),
                                                                  page=str(page),
                                                                  text_search=text_search,
                                                                  sort_property=sort_property,
                                                                  sort_order=sort_order)

    def delete_converter(self, converter_id: ConverterId):
        converter_id = self.get_id(converter_id)
        self.converter_controller.delete_converter_using_delete(converter_id=converter_id)

    def get_latest_converter_debug_input(self, converter_id: ConverterId):
        converter_id = self.get_id(converter_id)
        self.converter_controller.get_latest_converter_debug_input_using_get(converter_id=converter_id)

    def test_uplink_converter(self, input_params):
        return self.converter_controller.test_up_link_converter_using_post(input_params=input_params)

    def test_downlink_converter(self, input_params):
        return self.converter_controller.test_down_link_converter_using_post(input_params=input_params)

    def get_converters_by_ids(self, converter_ids):
        return self.converter_controller.get_converters_by_ids_using_get(converter_ids=converter_ids)

    """ Custom menu endpoints"""

    def get_custom_menu(self):
        return self.custom_menu_controller.get_custom_menu_using_get()

    def get_current_custom_menu(self):
        return self.custom_menu_controller.get_current_custom_menu_using_get()

    def save_custom_menu(self, custom_menu: CustomMenu):
        return self.custom_menu_controller.save_custom_menu_using_post(custom_menu=custom_menu)

    """ Custom translations endpoints"""

    def get_custom_translation(self):
        return self.custom_translation_controller.get_custom_translation_using_get()

    def get_current_custom_translation(self):
        return self.custom_translation_controller.get_current_custom_translation_using_get()

    def save_custom_translation(self, custom_translation: CustomTranslation):
        return self.custom_translation_controller.save_custom_translation_using_post(custom_translation=custom_translation)

    """Entity group endpoints"""

    def get_entity_group_by_id(self, entity_group_id: EntityGroupId):
        entity_group_id = self.get_id(entity_group_id)
        return self.entity_group_controller.get_entity_group_by_id_using_get(entity_group_id=entity_group_id)

    def save_entity_group(self, entity_group: EntityGroup):
        return self.entity_group_controller.save_entity_group_using_post(entity_group=entity_group)

    def delete_entity_group(self, entity_group_id: EntityGroupId):
        entity_group_id = self.get_id(entity_group_id)
        self.entity_group_controller.delete_entity_group_using_delete(entity_group_id=entity_group_id)

    def get_entity_groups_by_type(self, group_type):
        group_type = self.get_type(group_type)
        return self.entity_group_controller.get_entity_groups_by_type_using_get(group_type=group_type)

    def get_entity_groups_by_owner_and_type(self, owner_id, group_type):
        group_type = self.get_type(group_type)
        return self.entity_group_controller.get_entity_groups_by_owner_and_type_using_get(owner_type=owner_id.entity_type,
                                                                                          owner_id=owner_id.id,
                                                                                          group_type=group_type)

    def get_entity_group_all_by_owner_and_type(self, owner_id, group_type):
        group_type = self.get_type(group_type)
        return self.entity_group_controller.get_entity_group_all_by_owner_and_type_using_get(owner_type=owner_id.entity_type,
                                                                                             owner_id=owner_id.id,
                                                                                             group_type=group_type)

    def get_entity_group_info_by_owner_and_name_and_type(self, owner_id, group_type, group_name):
        group_type = self.get_type(group_type)
        return self.entity_group_controller.get_enitity_group_by_owner_and_name_and_type_using_get(owner_type=owner_id.entity_type,
                                                                                                   owner_id=owner_id.id,
                                                                                                   group_type=group_type,
                                                                                                   group_name=group_name)

    def add_entities_to_entity_group(self, entity_group_id: EntityGroupId, entity_ids):
        entity_group_id = self.get_id(entity_group_id)
        self.entity_group_controller.add_entities_to_entity_group_using_post(entity_group_id=entity_group_id,
                                                                             str_entity_ids=entity_ids)

    def remove_entities_from_entity_group(self, entity_group_id: EntityGroupId, entity_ids):
        entity_group_id = self.get_id(entity_group_id)
        self.entity_group_controller.remove_entities_from_entity_group_using_post(entity_group_id=entity_group_id,
                                                                                  str_entity_ids=entity_ids)

    def get_group_entity(self, entity_group_id: EntityGroupId, entity_id):
        entity_group_id = self.get_id(entity_group_id)
        entity_id = self.get_id(entity_id)
        return self.entity_group_controller.get_group_entity_using_get(entity_group_id=entity_group_id,
                                                                       entity_id=entity_id)

    def get_entities(self, entity_group_id: EntityGroupId, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None):
        entity_group_id = self.get_id(entity_group_id)
        return self.entity_group_controller.get_entities_using_get(entity_group_id=entity_group_id,
                                                                   page_size=str(page_size),
                                                                   page=str(page),
                                                                   text_search=text_search,
                                                                   sort_property=sort_property,
                                                                   sort_order=sort_order)

    def get_entity_groups_for_entity(self, entity_id: EntityId):
        entity_id = self.get_id(entity_id)
        return self.get_entity_groups_for_entity(entity_id=entity_id)

    def get_entity_groups_by_ids(self, entity_group_ids):
        return self.entity_group_controller.get_entity_groups_by_ids_using_get(entity_group_ids=entity_group_ids)

    def get_owners(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None):
        return self.entity_group_controller.get_owners_using_get(page_size=str(page_size),
                                                                 page=str(page),
                                                                 text_search=text_search,
                                                                 sort_property=sort_property,
                                                                 sort_order=sort_order)

    def make_entity_group_public(self, entity_group_id: EntityGroupId):
        entity_group_id = self.get_id(entity_group_id)
        self.entity_group_controller.make_entity_group_public_using_post(entity_group_id=entity_group_id)

    def make_entity_group_private(self, entity_group_id: EntityGroupId):
        entity_group_id = self.get_id(entity_group_id)
        self.entity_group_controller.make_entity_group_private_using_post(entity_group_id=entity_group_id)

    """ Group permissions endpoints"""

    def get_group_permission_by_id(self, group_permission_id: GroupPermissionId):
        group_permission_id = self.get_id(group_permission_id)
        return self.group_permission_controller.get_group_permission_by_id_using_get(group_permission_id=group_permission_id)

    def save_group_permission(self, group_permission: GroupPermission):
        return self.group_permission_controller.save_group_permission_using_post(group_permission=group_permission)

    def delete_group_permission(self, group_permission_id: GroupPermissionId):
        group_permission_id = self.get_id(group_permission_id)
        self.group_permission_controller.delete_group_permission_using_delete(group_permission_id=group_permission_id)

    def get_user_group_permissions(self, user_group_id):
        user_group_id = self.get_id(user_group_id)
        return self.group_permission_controller.get_user_group_permissions_using_get(user_group_id=user_group_id)

    def get_entity_group_permissions(self, entity_group_id: EntityGroupId):
        entity_group_id = self.get_id(entity_group_id)
        return self.group_permission_controller.get_entity_group_permissions_using_get(entity_group_id=entity_group_id)

    """ Integration endpoints. """

    def get_integration_by_id(self, integration_id: IntegrationId):
        integration_id = self.get_id(integration_id)
        return self.integration_controller.get_integration_by_id_using_get(integration_id=integration_id)

    def get_integration_by_routing_key(self, routing_key):
        return self.integration_controller.get_integration_by_routing_key_using_get(routing_key=routing_key)

    def save_integration(self, integration: Integration):
        return self.integration_controller.save_integration_using_post(integration=integration)

    def get_integrations(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None):
        return self.integration_controller.get_integrations_using_get(page_size=str(page_size),
                                                                      page=str(page),
                                                                      text_search=text_search,
                                                                      sort_property=sort_property,
                                                                      sort_order=sort_order)

    def delete_integration(self, integration_id: IntegrationId):
        integration_id = self.get_id(integration_id)
        self.integration_controller.delete_integration_using_delete(integration_id=integration_id)

    def get_integrations_by_ids(self, integration_ids):
        return self.integration_controller.get_integrations_by_ids_using_get(integration_ids=integration_ids)

    """ Device endpoints """


    def get_user_devices(self, device_type, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        return self.device_controller.get_user_devices_using_get(type=device_type,
                                                                 page_size=str(page_size),
                                                                 page=str(page),
                                                                 text_search=text_search,
                                                                 sort_property=sort_property,
                                                                 sort_order=sort_order, limit=str(limit))

    """ Entity view endpoints """

    def get_user_entity_views(self, entity_view_type, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        entity_view_type = self.get_type(entity_view_type)
        return self.entity_view_controller.get_user_entity_views_using_get(type=entity_view_type,
                                                                           page_size=str(page_size),
                                                                           page=str(page),
                                                                           text_search=text_search,
                                                                           sort_property=sort_property,
                                                                           sort_order=sort_order, limit=str(limit))

    def get_entity_views_by_ids(self, entity_view_ids):
        return self.entity_view_controller.get_entity_views_by_ids_using_get(entity_view_ids=entity_view_ids)


    """ Owner enpoints"""

    def change_owner_to_tenant(self, owner_id, entity_id):
        owner_id = self.get_id(owner_id)
        entity_type = self.get_type(entity_id)
        entity_id = self.get_id(entity_id)
        self.owner_controller.change_owner_to_tenant_using_post(owner_id=owner_id, entity_id=entity_id, entity_type=entity_type)

    def change_owner_to_customer(self, owner_id, entity_id):
        owner_id = self.get_id(owner_id)
        entity_type = self.get_type(entity_id)
        entity_id = self.get_id(entity_id)
        self.owner_controller.change_owner_to_customer_using_post(owner_id=owner_id, entity_id=entity_id, entity_type=entity_type)

    """ Reports endpoints """

    def download_dashboard_report(self, dashboard_id: DashboardId, report_params):
        dashboard_id = self.get_id(dashboard_id)
        return self.report_controller.download_dashboard_report_using_post(dashboard_id=dashboard_id, report_params=report_params)

    def download_test_report(self, report_config, reports_server_endpoint_url):
        return self.report_controller.download_test_report_using_post(report_config=report_config, reports_server_endpoint_url=reports_server_endpoint_url)

    """ Roles endpoints """

    def get_role_by_id(self, role_id: RoleId):
        role_id = self.get_id(role_id)
        return self.role_controller.get_role_by_id_using_get(role_id=role_id)

    def save_role(self, role: Role):
        return self.role_controller.save_role_using_post(role=role)

    def delete_role(self, role_id: RoleId):
        role_id = self.get_id(role_id)
        self.role_controller.delete_role_using_delete(role_id=role_id)

    def get_roles(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None):
        return self.role_controller.get_roles_using_get(page_size=str(page_size),
                                                        page=str(page),
                                                        text_search=text_search,
                                                        sort_property=sort_property,
                                                        sort_order=sort_order)

    def get_roles_by_ids(self, roles_ids):
        return self.role_controller.get_roles_by_ids_using_get(role_ids=roles_ids)

    def create_group_role(self, role_name, operations):
        role = Role(type="GROUP", name=role_name, permissions=operations)
        return self.save_role(role=role)

    """ User endpoints """

    def get_all_customer_users(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        return self.user_controller.get_all_customer_users_using_get(page_size=str(page_size),
                                                                     page=str(page),
                                                                     text_search=text_search,
                                                                     sort_property=sort_property,
                                                                     sort_order=sort_order, limit=str(limit))

    def get_user_users(self, page_size=10, page=0, text_search=None, sort_property=None, sort_order=None, limit=100000):
        return self.user_controller.get_user_users_using_get(page_size=page_size,
                                                             page=page,
                                                             text_search=text_search,
                                                             sort_property=sort_property,
                                                             sort_order=sort_order, limit=str(limit))

    def get_users_by_ids(self, user_ids):
        return self.user_controller.get_users_by_ids_using_get(user_ids=user_ids)

    def get_allowed_permissions(self):
        return self.user_permissions_controller.get_allowed_permissions_using_get()

    """ Rule engine endpoints"""

    def handle_rule_engine_request(self, request_body, entity_id=None, timeout=None):
        if entity_id is None and timeout is None:
            return self.rule_engine_controller.handle_rule_engine_request_using_post(request_body=request_body)
        elif timeout is None:
            entity_type = self.get_type(entity_id)
            entity_id = self.get_type(entity_id)
            return self.rule_engine_controller.handle_rule_engine_request_using_post2(entity_id=entity_id, entity_type=entity_type, request_body=request_body)
        else:
            entity_type = self.get_type(entity_id)
            entity_id = self.get_type(entity_id)
            return self.rule_engine_controller.handle_rule_engine_request_using_post1(entity_id=entity_id, entity_type=entity_type, request_body=request_body, timeout=timeout)

    """ Scheduler endpoints """

    def get_scheduler_event_info_by_id(self, scheduler_event_id: SchedulerEventId):
        scheduler_event_id = self.get_id(scheduler_event_id)
        return self.scheduler_event_controller.get_scheduler_event_info_by_id_using_get(scheduler_event_id=scheduler_event_id)

    def get_scheduler_event_by_id(self, scheduler_event_id: SchedulerEventId):
        scheduler_event_id = self.get_id(scheduler_event_id)
        return self.scheduler_event_controller.get_scheduler_event_by_id_using_get(scheduler_event_id=scheduler_event_id)

    def save_scheduler_event(self, scheduler_event: SchedulerEvent):
        return self.scheduler_event_controller.save_scheduler_event_using_post(scheduler_event=scheduler_event)

    def delete_scheduler_event(self, scheduler_event_id: SchedulerEventId):
        scheduler_event_id = self.get_id(scheduler_event_id)
        self.scheduler_event_controller.delete_scheduler_event_using_delete(scheduler_event_id=scheduler_event_id)

    def get_scheduler_events(self, type):
        return self.scheduler_event_controller.get_scheduler_events_using_get(type=type)

    def get_scheduler_events_by_ids(self, scheduler_event_ids):
        return self.scheduler_event_controller.get_scheduler_events_by_ids_using_get(scheduler_event_ids=scheduler_event_ids)

    """ Self registration endpoints. """

    def save_self_registration_params(self, self_registration_params: SelfRegistrationParams):
        return self.self_registration_controller.save_self_registration_params_using_post(self_registration_params=self_registration_params)

    def get_self_registration_params(self):
        return self.self_registration_controller.get_self_registration_params_using_get()

    def get_sign_up_self_registration_params(self):
        return self.self_registration_controller.get_sign_up_self_registration_params_using_get()

    def get_privacy_policy(self):
        return self.self_registration_controller.get_privacy_policy_using_get()

    """ Sign up endpoints. """

    def sign_up(self, sign_up_request: SignUpRequest):
        return self.sign_up_controller.sign_up_using_post(sign_up_request=sign_up_request)
    
    def resend_email_activation(self, email: str):
        return self.sign_up_controller.resend_email_activation_using_post(email=email)
    
    def activate_email(self, email_code):
        return self.sign_up_controller.activate_email_using_get(email_code=email_code)

    def activate_user_by_email_code(self, email_code):
        return self.sign_up_controller.activate_user_by_email_code_using_post(email_code=email_code)

    def privacy_policy_accepted(self):
        return self.sign_up_controller.privacy_policy_accepted_using_get()

    def accept_privace_policy(self):
        return self.sign_up_controller.accept_privacy_policy_using_post()

    """ White label endpoints. """

    def get_white_label_params(self, logo_image_checksum, favicon_checksum):
        return self.white_labeling_controller.get_white_label_params_using_get(logo_image_checksum=logo_image_checksum, favicon_checksum=favicon_checksum)

    def get_login_white_label_params(self, logo_image_checksum, favicon_checksum):
        return self.white_labeling_controller.get_login_white_label_params_using_get(logo_image_checksum=logo_image_checksum, favicon_checksum=favicon_checksum)

    def get_current_white_label_params(self):
        return self.white_labeling_controller.get_current_white_label_params_using_get()

    def get_current_login_white_label_params(self):
        return self.white_labeling_controller.get_current_login_white_label_params_using_get()

    def save_white_label_params(self, white_labeling_params: WhiteLabelingParams):
        return self.white_labeling_controller.save_white_label_params_using_post(white_labeling_params=white_labeling_params)

    def save_login_white_label_params(self, login_white_labeling_params: LoginWhiteLabelingParams):
        return self.white_labeling_controller.save_login_white_label_params_using_post(login_white_labeling_params=login_white_labeling_params)

    def preview_white_label_params(self, white_labeling_params: WhiteLabelingParams):
        return self.white_labeling_controller.preview_white_label_params_using_post(white_labeling_params=white_labeling_params)

    def is_white_labeling_allowed(self):
        return self.white_labeling_controller.is_white_labeling_allowed_using_get()

    def is_customer_white_labeling_allowed(self):
        return self.white_labeling_controller.is_customer_white_labeling_allowed_using_get()

    def __load_controllers(self):
        self.auth_controller = AuthControllerApi(self.api_client)
        self.admin_controller = AdminControllerApi(self.api_client)
        self.alarm_controller = AlarmControllerApi(self.api_client)
        self.asset_controller = AssetControllerApi(self.api_client)
        self.audit_log_controller = AuditLogControllerApi(self.api_client)
        self.blob_entity_controller = BlobEntityControllerApi(self.api_client)
        self.component_descriptor_controller = ComponentDescriptorControllerApi(self.api_client)
        self.converter_controller = ConverterControllerApi(self.api_client)
        self.custom_menu_controller = CustomMenuControllerApi(self.api_client)
        self.custom_translation_controller = CustomTranslationControllerApi(self.api_client)
        self.customer_controller = CustomerControllerApi(self.api_client)
        self.dashboard_controller = DashboardControllerApi(self.api_client)
        self.device_controller = DeviceControllerApi(self.api_client)
        self.entity_group_controller = EntityGroupControllerApi(self.api_client)
        self.entity_relation_controller = EntityRelationControllerApi(self.api_client)
        self.entity_view_controller = EntityViewControllerApi(self.api_client)
        self.event_controller = EventControllerApi(self.api_client)
        self.group_permission_controller = GroupPermissionControllerApi(self.api_client)
        self.http_integration_controller = HttpIntegrationControllerApi(self.api_client)
        self.integration_controller = IntegrationControllerApi(self.api_client)
        self.ocean_connect_integration_controller = OceanConnectIntegrationControllerApi(self.api_client)
        self.owner_controller = OwnerControllerApi(self.api_client)
        self.queue_controller = QueueControllerApi(self.api_client)
        self.report_controller = ReportControllerApi(self.api_client)
        self.role_controller = RoleControllerApi(self.api_client)
        self.rpc_controller = RpcControllerApi(self.api_client)
        self.rule_chain_controller = RuleChainControllerApi(self.api_client)
        self.rule_engine_controller = RuleEngineControllerApi(self.api_client)
        self.scheduler_event_controller = SchedulerEventControllerApi(self.api_client)
        self.self_registration_controller = SelfRegistrationControllerApi(self.api_client)
        self.sigfox_integration_controller = SigFoxIntegrationControllerApi(self.api_client)
        self.sign_up_controller = SignUpControllerApi(self.api_client)
        self.tmobile_iot_cdp_integration_controller = TMobileIotCdpIntegrationControllerApi(self.api_client)
        self.telemetry_controller = TelemetryControllerApi(self.api_client)
        self.tenant_controller = TenantControllerApi(self.api_client)
        self.thingpark_integration_controller = ThingParkIntegrationControllerApi(self.api_client)
        self.trail_controller = TrailControllerApi(self.api_client)
        self.user_controller = UserControllerApi(self.api_client)
        self.user_permissions_controller = UserPermissionsControllerApi(self.api_client)
        self.white_labeling_controller = WhiteLabelingControllerApi(self.api_client)
        self.widget_type_controller = WidgetTypeControllerApi(self.api_client)
        self.widgets_bundle_controller = WidgetsBundleControllerApi(self.api_client)

    @staticmethod
    def get_type(type):
        return type.entity_type if hasattr(type, "entity_type") else type

    @staticmethod
    def get_id(id):
        return id.id if hasattr(id, "id") else id



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from tb_rest_client.api_client import ApiClient


class DashboardControllerApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_dashboard_customers_using_post(self, dashboard_id, str_customer_ids, **kwargs):  # noqa: E501
        """addDashboardCustomers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.add_dashboard_customers_using_post(dashboard_id, str_customer_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :param list[str] str_customer_ids: strCustomerIds (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, **kwargs)  # noqa: E501
        else:
            (data) = self.add_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, **kwargs)  # noqa: E501
            return data

    def add_dashboard_customers_using_post_with_http_info(self, dashboard_id, str_customer_ids, **kwargs):  # noqa: E501
        """addDashboardCustomers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.add_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :param list[str] str_customer_ids: strCustomerIds (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id', 'str_customer_ids']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `add_dashboard_customers_using_post`")  # noqa: E501
        # verify the required parameter 'str_customer_ids' is set
        if ('str_customer_ids' not in params or
                params['str_customer_ids'] is None):
            raise ValueError("Missing the required parameter `str_customer_ids` when calling `add_dashboard_customers_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'str_customer_ids' in params:
            body_params = params['str_customer_ids']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/{dashboardId}/customers/add', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def assign_dashboard_to_customer_using_post(self, customer_id, dashboard_id, **kwargs):  # noqa: E501
        """assignDashboardToCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.assign_dashboard_to_customer_using_post(customer_id, dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.assign_dashboard_to_customer_using_post_with_http_info(customer_id, dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.assign_dashboard_to_customer_using_post_with_http_info(customer_id, dashboard_id, **kwargs)  # noqa: E501
            return data

    def assign_dashboard_to_customer_using_post_with_http_info(self, customer_id, dashboard_id, **kwargs):  # noqa: E501
        """assignDashboardToCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.assign_dashboard_to_customer_using_post_with_http_info(customer_id, dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['customer_id', 'dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'customer_id' is set
        if ('customer_id' not in params or
                params['customer_id'] is None):
            raise ValueError("Missing the required parameter `customer_id` when calling `assign_dashboard_to_customer_using_post`")  # noqa: E501
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `assign_dashboard_to_customer_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'customer_id' in params:
            path_params['customerId'] = params['customer_id']  # noqa: E501
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/{customerId}/dashboard/{dashboardId}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def assign_dashboard_to_public_customer_using_post(self, dashboard_id, **kwargs):  # noqa: E501
        """assignDashboardToPublicCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.assign_dashboard_to_public_customer_using_post(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.assign_dashboard_to_public_customer_using_post_with_http_info(dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.assign_dashboard_to_public_customer_using_post_with_http_info(dashboard_id, **kwargs)  # noqa: E501
            return data

    def assign_dashboard_to_public_customer_using_post_with_http_info(self, dashboard_id, **kwargs):  # noqa: E501
        """assignDashboardToPublicCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.assign_dashboard_to_public_customer_using_post_with_http_info(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `assign_dashboard_to_public_customer_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/public/dashboard/{dashboardId}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_dashboard_using_delete(self, dashboard_id, **kwargs):  # noqa: E501
        """deleteDashboard  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.delete_dashboard_using_delete(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_dashboard_using_delete_with_http_info(dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_dashboard_using_delete_with_http_info(dashboard_id, **kwargs)  # noqa: E501
            return data

    def delete_dashboard_using_delete_with_http_info(self, dashboard_id, **kwargs):  # noqa: E501
        """deleteDashboard  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.delete_dashboard_using_delete_with_http_info(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `delete_dashboard_using_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/{dashboardId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_customer_dashboards_using_get(self, customer_id, limit, **kwargs):  # noqa: E501
        """getCustomerDashboards  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_customer_dashboards_using_get(customer_id, limit, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str limit: limit (required)
        :param int start_time: startTime
        :param int end_time: endTime
        :param bool asc_order: ascOrder
        :param str offset: offset
        :return: TimePageDataDashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_customer_dashboards_using_get_with_http_info(customer_id, limit, **kwargs)  # noqa: E501
        else:
            (data) = self.get_customer_dashboards_using_get_with_http_info(customer_id, limit, **kwargs)  # noqa: E501
            return data

    def get_customer_dashboards_using_get_with_http_info(self, customer_id, limit, **kwargs):  # noqa: E501
        """getCustomerDashboards  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_customer_dashboards_using_get_with_http_info(customer_id, limit, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str limit: limit (required)
        :param int start_time: startTime
        :param int end_time: endTime
        :param bool asc_order: ascOrder
        :param str offset: offset
        :return: TimePageDataDashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['customer_id', 'limit', 'start_time', 'end_time', 'asc_order', 'offset']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'customer_id' is set
        if ('customer_id' not in params or
                params['customer_id'] is None):
            raise ValueError("Missing the required parameter `customer_id` when calling `get_customer_dashboards_using_get`")  # noqa: E501
        # verify the required parameter 'limit' is set
        if ('limit' not in params or
                params['limit'] is None):
            raise ValueError("Missing the required parameter `limit` when calling `get_customer_dashboards_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'customer_id' in params:
            path_params['customerId'] = params['customer_id']  # noqa: E501

        query_params = []
        if 'start_time' in params:
            query_params.append(('startTime', params['start_time']))  # noqa: E501
        if 'end_time' in params:
            query_params.append(('endTime', params['end_time']))  # noqa: E501
        if 'asc_order' in params:
            query_params.append(('ascOrder', params['asc_order']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/{customerId}/dashboards{?startTime,endTime,ascOrder,offset,limit}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='TimePageDataDashboardInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_dashboard_by_id_using_get(self, dashboard_id, **kwargs):  # noqa: E501
        """getDashboardById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_dashboard_by_id_using_get(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_dashboard_by_id_using_get_with_http_info(dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_dashboard_by_id_using_get_with_http_info(dashboard_id, **kwargs)  # noqa: E501
            return data

    def get_dashboard_by_id_using_get_with_http_info(self, dashboard_id, **kwargs):  # noqa: E501
        """getDashboardById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_dashboard_by_id_using_get_with_http_info(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `get_dashboard_by_id_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/{dashboardId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_dashboard_info_by_id_using_get(self, dashboard_id, **kwargs):  # noqa: E501
        """getDashboardInfoById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_dashboard_info_by_id_using_get(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: DashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_dashboard_info_by_id_using_get_with_http_info(dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_dashboard_info_by_id_using_get_with_http_info(dashboard_id, **kwargs)  # noqa: E501
            return data

    def get_dashboard_info_by_id_using_get_with_http_info(self, dashboard_id, **kwargs):  # noqa: E501
        """getDashboardInfoById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_dashboard_info_by_id_using_get_with_http_info(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: DashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `get_dashboard_info_by_id_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/info/{dashboardId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='DashboardInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_max_datapoints_limit_using_get(self, **kwargs):  # noqa: E501
        """getMaxDatapointsLimit  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_max_datapoints_limit_using_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_max_datapoints_limit_using_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_max_datapoints_limit_using_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_max_datapoints_limit_using_get_with_http_info(self, **kwargs):  # noqa: E501
        """getMaxDatapointsLimit  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_max_datapoints_limit_using_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/maxDatapointsLimit', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_server_time_using_get(self, **kwargs):  # noqa: E501
        """getServerTime  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_server_time_using_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_server_time_using_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_server_time_using_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_server_time_using_get_with_http_info(self, **kwargs):  # noqa: E501
        """getServerTime  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_server_time_using_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/serverTime', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_tenant_dashboards_using_get(self, limit, **kwargs):  # noqa: E501
        """getTenantDashboards  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_dashboards_using_get(limit, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str limit: limit (required)
        :param str text_search: textSearch
        :param str id_offset: idOffset
        :param str text_offset: textOffset
        :return: TextPageDataDashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_tenant_dashboards_using_get_with_http_info(limit, **kwargs)  # noqa: E501
        else:
            (data) = self.get_tenant_dashboards_using_get_with_http_info(limit, **kwargs)  # noqa: E501
            return data

    def get_tenant_dashboards_using_get_with_http_info(self, limit, **kwargs):  # noqa: E501
        """getTenantDashboards  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_dashboards_using_get_with_http_info(limit, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str limit: limit (required)
        :param str text_search: textSearch
        :param str id_offset: idOffset
        :param str text_offset: textOffset
        :return: TextPageDataDashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['limit', 'text_search', 'id_offset', 'text_offset']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'limit' is set
        if ('limit' not in params or
                params['limit'] is None):
            raise ValueError("Missing the required parameter `limit` when calling `get_tenant_dashboards_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'text_search' in params:
            query_params.append(('textSearch', params['text_search']))  # noqa: E501
        if 'id_offset' in params:
            query_params.append(('idOffset', params['id_offset']))  # noqa: E501
        if 'text_offset' in params:
            query_params.append(('textOffset', params['text_offset']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/tenant/dashboards{?textSearch,idOffset,textOffset,limit}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='TextPageDataDashboardInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_tenant_dashboards_using_get1(self, tenant_id, limit, **kwargs):  # noqa: E501
        """getTenantDashboards  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_dashboards_using_get1(tenant_id, limit, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str tenant_id: tenantId (required)
        :param str limit: limit (required)
        :param str text_search: textSearch
        :param str id_offset: idOffset
        :param str text_offset: textOffset
        :return: TextPageDataDashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_tenant_dashboards_using_get1_with_http_info(tenant_id, limit, **kwargs)  # noqa: E501
        else:
            (data) = self.get_tenant_dashboards_using_get1_with_http_info(tenant_id, limit, **kwargs)  # noqa: E501
            return data

    def get_tenant_dashboards_using_get1_with_http_info(self, tenant_id, limit, **kwargs):  # noqa: E501
        """getTenantDashboards  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_dashboards_using_get1_with_http_info(tenant_id, limit, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str tenant_id: tenantId (required)
        :param str limit: limit (required)
        :param str text_search: textSearch
        :param str id_offset: idOffset
        :param str text_offset: textOffset
        :return: TextPageDataDashboardInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['tenant_id', 'limit', 'text_search', 'id_offset', 'text_offset']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'tenant_id' is set
        if ('tenant_id' not in params or
                params['tenant_id'] is None):
            raise ValueError("Missing the required parameter `tenant_id` when calling `get_tenant_dashboards_using_get1`")  # noqa: E501
        # verify the required parameter 'limit' is set
        if ('limit' not in params or
                params['limit'] is None):
            raise ValueError("Missing the required parameter `limit` when calling `get_tenant_dashboards_using_get1`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'tenant_id' in params:
            path_params['tenantId'] = params['tenant_id']  # noqa: E501

        query_params = []
        if 'text_search' in params:
            query_params.append(('textSearch', params['text_search']))  # noqa: E501
        if 'id_offset' in params:
            query_params.append(('idOffset', params['id_offset']))  # noqa: E501
        if 'text_offset' in params:
            query_params.append(('textOffset', params['text_offset']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/tenant/{tenantId}/dashboards{?textSearch,idOffset,textOffset,limit}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='TextPageDataDashboardInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_dashboard_customers_using_post(self, dashboard_id, str_customer_ids, **kwargs):  # noqa: E501
        """removeDashboardCustomers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.remove_dashboard_customers_using_post(dashboard_id, str_customer_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :param list[str] str_customer_ids: strCustomerIds (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.remove_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, **kwargs)  # noqa: E501
            return data

    def remove_dashboard_customers_using_post_with_http_info(self, dashboard_id, str_customer_ids, **kwargs):  # noqa: E501
        """removeDashboardCustomers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.remove_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :param list[str] str_customer_ids: strCustomerIds (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id', 'str_customer_ids']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `remove_dashboard_customers_using_post`")  # noqa: E501
        # verify the required parameter 'str_customer_ids' is set
        if ('str_customer_ids' not in params or
                params['str_customer_ids'] is None):
            raise ValueError("Missing the required parameter `str_customer_ids` when calling `remove_dashboard_customers_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'str_customer_ids' in params:
            body_params = params['str_customer_ids']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/{dashboardId}/customers/remove', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def save_dashboard_using_post(self, dashboard, **kwargs):  # noqa: E501
        """saveDashboard  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.save_dashboard_using_post(dashboard, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Dashboard dashboard: dashboard (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.save_dashboard_using_post_with_http_info(dashboard, **kwargs)  # noqa: E501
        else:
            (data) = self.save_dashboard_using_post_with_http_info(dashboard, **kwargs)  # noqa: E501
            return data

    def save_dashboard_using_post_with_http_info(self, dashboard, **kwargs):  # noqa: E501
        """saveDashboard  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.save_dashboard_using_post_with_http_info(dashboard, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Dashboard dashboard: dashboard (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard' is set
        if ('dashboard' not in params or
                params['dashboard'] is None):
            raise ValueError("Missing the required parameter `dashboard` when calling `save_dashboard_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'dashboard' in params:
            body_params = params['dashboard']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def unassign_dashboard_from_customer_using_delete(self, customer_id, dashboard_id, **kwargs):  # noqa: E501
        """unassignDashboardFromCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.unassign_dashboard_from_customer_using_delete(customer_id, dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.unassign_dashboard_from_customer_using_delete_with_http_info(customer_id, dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.unassign_dashboard_from_customer_using_delete_with_http_info(customer_id, dashboard_id, **kwargs)  # noqa: E501
            return data

    def unassign_dashboard_from_customer_using_delete_with_http_info(self, customer_id, dashboard_id, **kwargs):  # noqa: E501
        """unassignDashboardFromCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.unassign_dashboard_from_customer_using_delete_with_http_info(customer_id, dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['customer_id', 'dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'customer_id' is set
        if ('customer_id' not in params or
                params['customer_id'] is None):
            raise ValueError("Missing the required parameter `customer_id` when calling `unassign_dashboard_from_customer_using_delete`")  # noqa: E501
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `unassign_dashboard_from_customer_using_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'customer_id' in params:
            path_params['customerId'] = params['customer_id']  # noqa: E501
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/{customerId}/dashboard/{dashboardId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def unassign_dashboard_from_public_customer_using_delete(self, dashboard_id, **kwargs):  # noqa: E501
        """unassignDashboardFromPublicCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.unassign_dashboard_from_public_customer_using_delete(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.unassign_dashboard_from_public_customer_using_delete_with_http_info(dashboard_id, **kwargs)  # noqa: E501
        else:
            (data) = self.unassign_dashboard_from_public_customer_using_delete_with_http_info(dashboard_id, **kwargs)  # noqa: E501
            return data

    def unassign_dashboard_from_public_customer_using_delete_with_http_info(self, dashboard_id, **kwargs):  # noqa: E501
        """unassignDashboardFromPublicCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.unassign_dashboard_from_public_customer_using_delete_with_http_info(dashboard_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `unassign_dashboard_from_public_customer_using_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/public/dashboard/{dashboardId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_dashboard_customers_using_post(self, dashboard_id, str_customer_ids, **kwargs):  # noqa: E501
        """updateDashboardCustomers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.update_dashboard_customers_using_post(dashboard_id, str_customer_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :param list[str] str_customer_ids: strCustomerIds (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, **kwargs)  # noqa: E501
        else:
            (data) = self.update_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, **kwargs)  # noqa: E501
            return data

    def update_dashboard_customers_using_post_with_http_info(self, dashboard_id, str_customer_ids, **kwargs):  # noqa: E501
        """updateDashboardCustomers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.update_dashboard_customers_using_post_with_http_info(dashboard_id, str_customer_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str dashboard_id: dashboardId (required)
        :param list[str] str_customer_ids: strCustomerIds (required)
        :return: Dashboard
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['dashboard_id', 'str_customer_ids']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'dashboard_id' is set
        if ('dashboard_id' not in params or
                params['dashboard_id'] is None):
            raise ValueError("Missing the required parameter `dashboard_id` when calling `update_dashboard_customers_using_post`")  # noqa: E501
        # verify the required parameter 'str_customer_ids' is set
        if ('str_customer_ids' not in params or
                params['str_customer_ids'] is None):
            raise ValueError("Missing the required parameter `str_customer_ids` when calling `update_dashboard_customers_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'dashboard_id' in params:
            path_params['dashboardId'] = params['dashboard_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'str_customer_ids' in params:
            body_params = params['str_customer_ids']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/dashboard/{dashboardId}/customers', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Dashboard',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

from __future__ import absolute_import

# flake8: noqa

# import apis into api_pe package
from .admin_controller_api import AdminControllerApi
from .alarm_controller_api import AlarmControllerApi
from .asset_controller_api import AssetControllerApi
from .audit_log_controller_api import AuditLogControllerApi
from .blob_entity_controller_api import BlobEntityControllerApi
from .converter_controller_api import ConverterControllerApi
from .custom_menu_controller_api import CustomMenuControllerApi
from .custom_translation_controller_api import CustomTranslationControllerApi
from .customer_controller_api import CustomerControllerApi
from .dashboard_controller_api import DashboardControllerApi
from .device_controller_api import DeviceControllerApi
from .entity_group_controller_api import EntityGroupControllerApi
from .entity_view_controller_api import EntityViewControllerApi
from .event_controller_api import EventControllerApi
from .group_permission_controller_api import GroupPermissionControllerApi
from .http_integration_controller_api import HttpIntegrationControllerApi
from .integration_controller_api import IntegrationControllerApi
from .ocean_connect_integration_controller_api import OceanConnectIntegrationControllerApi
from .owner_controller_api import OwnerControllerApi
from .report_controller_api import ReportControllerApi
from .role_controller_api import RoleControllerApi
from .rule_chain_controller_api import RuleChainControllerApi
from .rule_engine_controller_api import RuleEngineControllerApi
from .scheduler_event_controller_api import SchedulerEventControllerApi
from .self_registration_controller_api import SelfRegistrationControllerApi
from .sig_fox_integration_controller_api import SigFoxIntegrationControllerApi
from .sign_up_controller_api import SignUpControllerApi
from .t_mobile_iot_cdp_integration_controller_api import TMobileIotCdpIntegrationControllerApi
from .tenant_controller_api import TenantControllerApi
from .thing_park_integration_controller_api import ThingParkIntegrationControllerApi
from .trail_controller_api import TrailControllerApi
from .user_controller_api import UserControllerApi
from .user_permissions_controller_api import UserPermissionsControllerApi
from .white_labeling_controller_api import WhiteLabelingControllerApi
from .widgets_bundle_controller_api import WidgetsBundleControllerApi



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

import pprint
import re  # noqa: F401

import six

from tb_rest_client.models.models_ce import AuditLog


class AuditLog(AuditLog):

    def __init__(self, action_data=None, action_failure_details=None, action_status=None, action_type=None, created_time=None, customer_id=None, entity_id=None, entity_name=None, id=None, tenant_id=None, user_id=None, user_name=None):  # noqa: E501
        super().__init__(action_data, action_failure_details, action_status, action_type, created_time, customer_id, entity_id, entity_name, id, tenant_id, user_id, user_name)
        self._action_data = None
        if action_data is not None:
            self.action_data = action_data

    @property
    def action_type(self):
        """Gets the action_type of this AuditLog.  # noqa: E501


        :return: The action_type of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._action_type

    @action_type.setter
    def action_type(self, action_type):
        """Sets the action_type of this AuditLog.


        :param action_type: The action_type of this AuditLog.  # noqa: E501
        :type: str
        """
        allowed_values = ["ADDED", "DELETED", "UPDATED", "ATTRIBUTES_UPDATED", "ATTRIBUTES_DELETED", "TIMESERIES_DELETED", "RPC_CALL", "CREDENTIALS_UPDATED", "ASSIGNED_TO_CUSTOMER", "UNASSIGNED_FROM_CUSTOMER", "CHANGE_OWNER", "ACTIVATED", "SUSPENDED", "CREDENTIALS_READ", "ATTRIBUTES_READ", "RELATION_ADD_OR_UPDATE", "RELATION_DELETED", "RELATIONS_DELETED", "ALARM_ACK", "ALARM_CLEAR", "ADDED_TO_ENTITY_GROUP", "REMOVED_FROM_ENTITY_GROUP", "REST_API_RULE_ENGINE_CALL", "MADE_PUBLIC", "MADE_PRIVATE", "LOGIN", "LOGOUT", "LOCKOUT"]  # noqa: E501
        if action_type not in allowed_values:
            raise ValueError(
                "Invalid value for `action_type` ({0}), must be one of {1}"  # noqa: E501
                .format(action_type, allowed_values)
            )

        self._action_type = action_type
